Index: cgi/avail.c
===================================================================
RCS file: /cvsroot/nagios/nagios/cgi/avail.c,v
retrieving revision 1.23
diff -u -r1.23 avail.c
--- cgi/avail.c	21 Jan 2004 04:56:09 -0000	1.23
+++ cgi/avail.c	17 Sep 2004 11:58:06 -0000
@@ -41,9 +41,18 @@
 extern hostgroup *hostgroup_list;
 extern servicegroup *servicegroup_list;
 extern service   *service_list;
+extern timeperiod      *timeperiod_list;
 
 extern int       log_rotation_method;
 
+#ifndef max
+#define max(a,b)  (((a) > (b)) ? (a) : (b))
+#endif
+#ifndef min
+#define min(a,b)  (((a) < (b)) ? (a) : (b))
+#endif
+
+
 
 /* output types */
 #define HTML_OUTPUT             0
@@ -104,7 +113,7 @@
 #define MAX_ARCHIVE_BACKTRACKS	60
 
 authdata current_authdata;
-
+timeperiod *current_timeperiod = NULL;
 typedef struct archived_state_struct{
 	time_t  time_stamp;
 	int     entry_type;
@@ -237,6 +246,9 @@
 void document_footer(void);
 int process_cgivars(void);
 
+unsigned long calculate_total_time(time_t start_time, time_t end_time);
+
+
 int backtrack_archives=2;
 int earliest_archive=0;
 
@@ -600,6 +612,7 @@
 
 	/* step 3 - ask user for report date range */
 	if(get_date_parts==TRUE){
+		timeperiod *temp_timeperiod;
 
 		time(&current_time);
 		t=localtime(&current_time);
@@ -700,6 +713,22 @@
 
 		printf("<tr><td colspan=2><br></td></tr>\n");
 
+		// Added new dropdown to select timeperiods.
+
+		printf("<tr>");
+		printf("<td valign=top class='reportSelectSubTitle'>Report time Period:</td>\n");
+		printf("<td valign=top align=left class='optBoxItem'>\n");
+		printf("<select name='rpttimeperiod'>\n");
+		printf("<option value=\"\">None\n");
+		/* check all the time periods... */
+		for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){
+			printf("<option value=%s>%s\n", url_encode(temp_timeperiod->name), temp_timeperiod->name);
+		}
+		printf("</select>\n");
+		printf("</td>\n");
+		printf("</tr>\n");
+		printf("<tr><td colspan=2><br></td></tr>\n");
+
 		printf("<tr><td class='reportSelectSubTitle' align=right>Assume Initial States:</td>\n");
 		printf("<td class='reportSelectItem'>\n");
 		printf("<select name='assumeinitialstates'>\n");
@@ -735,7 +764,7 @@
 			printf("<option value=%d>Host Unreachable\n",AS_HOST_UNREACHABLE);
 			printf("</select>\n");
 			printf("</td></tr>\n");
-		        }
+		}
 
 		printf("<tr><td class='reportSelectSubTitle' align=right>First Assumed Service State:</td>\n");
 		printf("<td class='reportSelectItem'>\n");
@@ -1555,13 +1584,29 @@
 			if(variables[x]==NULL){
 				error=TRUE;
 				break;
-			        }
+			}
 
 			if(!strcmp(variables[x],"yes"))
 				show_scheduled_downtime=TRUE;
 			else
 				show_scheduled_downtime=FALSE;
-		        }
+		}
+		/* we found the report timeperiod option */
+		else if(!strcmp(variables[x],"rpttimeperiod")){
+			timeperiod *temp_timeperiod;
+			x++;
+			if(variables[x]==NULL){
+				error=TRUE;
+				break;
+			}
+
+			for(temp_timeperiod=timeperiod_list;temp_timeperiod!=NULL;temp_timeperiod=temp_timeperiod->next){
+				if (!strcmp(url_encode(temp_timeperiod->name),variables[x])) {
+					current_timeperiod = temp_timeperiod;
+					break;
+				}
+			}
+		}
 
 	        }
 
@@ -1900,7 +1945,6 @@
         }
 
 
-
 /* computes availability times */
 void compute_subject_availability_times(int first_state,int last_state,time_t real_start_time,time_t start_time,time_t end_time,avail_subject *subject, archived_state *as){
 	int start_state;
@@ -1928,8 +1972,62 @@
 	if(end_time<t1)
 		return;
 
-	/* calculate time in this state */
-	state_duration=(unsigned long)(end_time-start_time);
+// MickeM Attempt to handle the current time_period (if any)
+	if (current_timeperiod) {
+		struct tm *t;
+		t=localtime((time_t *)&start_time);
+		state_duration = 0;
+
+		/* calculate the start of the day (midnight, 00:00 hours) */
+		t->tm_sec=0;
+		t->tm_min=0;
+		t->tm_hour=0;
+		unsigned long midnight_today=(unsigned long)mktime(t);
+		int weekday = t->tm_wday;
+
+		while (midnight_today<end_time) {
+			timerange *temp_timerange;
+			unsigned long temp_duration = 0;
+			unsigned long temp_end = min(86400, end_time - midnight_today);
+			unsigned long temp_start = 0;
+			if (start_time > midnight_today)
+				temp_start = start_time - midnight_today;
+#ifdef DEBUG
+			printf("<b>Matching: %ld -> %ld. (%ld -> %ld)</b><br>\n",temp_start, temp_end, midnight_today+temp_start, midnight_today+temp_end);
+#endif
+			/* check all time ranges for this day of the week */
+			for(temp_timerange=current_timeperiod->days[weekday];
+				temp_timerange!=NULL;
+				temp_timerange=temp_timerange->next){
+					unsigned long start, end;
+#ifdef DEBUG
+					printf("<li>Matching in timerang[%d]: %d -> %d (%ld -> %ld)<br>\n",weekday, temp_timerange->range_start, temp_timerange->range_end,temp_start, temp_end);
+#endif
+					start = max(temp_timerange->range_start, temp_start);
+					end = min(temp_timerange->range_end, temp_end);
+
+					if (start < end)  {
+						temp_duration += end-start;
+#ifdef DEBUG
+						printf("<li>Matched time: %ld -> %ld = %d<br>\n",start, end, temp_duration);
+#endif
+					} 
+#ifdef DEBUG
+					else {
+						printf("<li>Ignored time: %ld -> %ld<br>\n",start, end);
+					}
+#endif
+				}
+				state_duration+=temp_duration;
+				temp_start = 0;
+				midnight_today += 86400;
+				if (++weekday>6)
+					weekday = 0;
+		}
+	} else {
+		/* calculate time in this state */
+		state_duration = (unsigned long)(end_time-start_time);
+	}
 
 	/* can't graph if we don't have data... */
 	if(first_state==AS_NO_DATA || last_state==AS_NO_DATA){
@@ -3311,7 +3409,8 @@
 	if(is_authorized_for_hostgroup(hg,&current_authdata)==FALSE)
 		return;
 
-	total_time=t2-t1;
+	/* Calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1, t2);
 
 	printf("<BR><BR>\n");
 	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Hostgroup '%s' Host State Breakdowns:</DIV>\n",hg->group_name);
@@ -3472,7 +3571,8 @@
 	if(is_authorized_for_servicegroup(sg,&current_authdata)==FALSE)
 		return;
 
-	total_time=t2-t1;
+	/* Calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1, t2);
 
 	printf("<BR><BR>\n");
 	printf("<DIV ALIGN=CENTER CLASS='dataTitle'>Servicegroup '%s' Host State Breakdowns:</DIV>\n",sg->group_name);
@@ -3738,9 +3838,12 @@
 	char *bgclass="";
 	int odd=1;
 
+	/* Calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1, t2);
 
-	total_time=t2-t1;
-
+#ifdef DEBUG
+	printf("Total time: '%ld' seconds<br>\n",total_time);
+#endif
 
 	/* show data for a specific host */
 	if(show_all_hosts==FALSE){
@@ -4215,7 +4318,8 @@
 	char last_host[128]="";
 
 
-	total_time=t2-t1;
+	/* Calculate total time during period based on timeperiod used for reporting */
+	total_time=calculate_total_time(t1, t2);
 
 	/* we're only getting data for one service */
 	if(show_all_services==FALSE){
@@ -4584,6 +4688,8 @@
 	if(full_log_entries==TRUE)
 		printf("&full_log_entries");
 	printf("&showscheduleddowntime=%s",(show_scheduled_downtime==TRUE)?"yes":"no");
+	if (current_timeperiod)
+		printf("&rpttimeperiod=%s", url_encode(current_timeperiod->name));
 	printf("'>%s</a>",label);
 
 	return;
@@ -4606,6 +4712,8 @@
 	if(full_log_entries==TRUE)
 		printf("&full_log_entries");
 	printf("&showscheduleddowntime=%s",(show_scheduled_downtime==TRUE)?"yes":"no");
+	if (current_timeperiod)
+		printf("&rpttimeperiod=%s", url_encode(current_timeperiod->name));
 	printf("'>%s</a>",label);
 
 	return;
@@ -4620,3 +4728,50 @@
 	return;
         }
 
+
+
+unsigned long calculate_total_time(time_t start_time, time_t end_time) {
+	// MickeM Attempt to handle the current time_period
+	if (current_timeperiod) {
+		// "A day" is 86400 seconds
+		struct tm *t=localtime(&start_time);
+
+		// calculate the start of the day (midnight, 00:00 hours) 
+		t->tm_sec=0;
+		t->tm_min=0;
+		t->tm_hour=0;
+		unsigned long midnight_today=(unsigned long)mktime(t);
+		int weekday = t->tm_wday;
+
+		unsigned long total_time = 0;
+		while (midnight_today<end_time) {
+			timerange *temp_timerange;
+			unsigned long temp_duration = 0;
+			unsigned long temp_end = min(86400, t2 - midnight_today);
+			unsigned long temp_start = 0;
+			if (t1 > midnight_today)
+				temp_start = t1 - midnight_today;
+
+			// check all time ranges for this day of the week 
+			for(temp_timerange=current_timeperiod->days[weekday];
+				temp_timerange!=NULL;
+				temp_timerange=temp_timerange->next){
+					unsigned long start = max(temp_timerange->range_start, temp_start);
+					unsigned long end = min(temp_timerange->range_end, temp_end);
+#ifdef DEBUG
+					printf("<li>Matching in timerange[%d]: %d -> %d (%ld -> %ld) %d -> %d = %ld<br>\n",weekday, temp_timerange->range_start, temp_timerange->range_end,temp_start, temp_end, start, end, end-start);
+#endif
+					if (end > start)
+						temp_duration += end-start;
+				}
+				total_time+=temp_duration;
+				temp_start = 0;
+				midnight_today += 86400;
+				if (++weekday>6)
+					weekday = 0;
+		}
+		return total_time;
+	} else {
+		return end_time-start_time;
+	}
+}
